@@ -146,13 +146,16 @@ class NumerousBufferedWriter:
         self._init_buffer()
 
     def close(self):
-        self.flush()
-        self.closed = False
-        self.write_queue.put('STOP')
+        if not self.closed:
+            self.flush()
 
-        self.writer_thread.join()
+            self.write_queue.put('STOP')
 
-        self._writer_closed(self.scenario)
+            self.writer_thread.join()
+
+            self._writer_closed(self.scenario)
+
+        self.closed = True
 
 class NumerousLogHandler:
     def __init__(self, client):
@@ -164,8 +167,7 @@ class NumerousLogHandler:
     #TODO make a handler thread to schedule update to server!
 
 class NumerousClient:
-    def __init__(self, job_id=None, project=None, scenario=None, server=None, port=None, token=None, clear_data=None, no_log=False, instance_id=None):
-
+    def __init__(self, job_id=None, project=None, scenario=None, server=None, port=None, secure=None, token=None, clear_data=None, no_log=False, instance_id=None):
 
         self.job_states = JobStatus
         self._project = get_env(project, 'NUMEROUS_PROJECT')
@@ -179,8 +181,8 @@ class NumerousClient:
             log.debug(f'No execution id found - generating new: {self._execution_id}')
 
         self._complete = True
-
-        channel = self._init_channel(server=server, port=port, instance_id=instance_id)
+        self.writers = []
+        channel = self._init_channel(server=server, port=port, secure=secure, instance_id=instance_id)
         stub = spm_pb2_grpc.SPMStub(channel)
 
         # TODO: Maybe reconsider
@@ -218,14 +220,15 @@ class NumerousClient:
     def complete_execution(self):
         self._base_client.CompleteExecution(spm_pb2.ExecutionMessage(project_id=self._project, scenario_id=self._scenario, job_id=self._job_id, execution_id=self._execution_id))
 
-    def _init_channel(self, server, port, instance_id=None):
-        secure_channel = os.getenv('SECURE_CHANNEL', False)
+    def _init_channel(self, server, port, secure=None, instance_id=None):
+        secure_channel = get_env(secure, 'SECURE_CHANNEL')
         server = get_env(server, 'NUMEROUS_API_SERVER')
         port = get_env(port, 'NUMEROUS_API_PORT')
 
         log.info(f"Client connecting to: {server}:{port}, using secure channel: {secure_channel}")
-
+        print(server,':',port,' secure: ', str(secure_channel))
         if str(secure_channel) == 'True':
+            print('secure connect!')
             with open(f'{os.path.dirname(os.path.abspath(__file__))}/certs/server.crt', 'rb') as f:
                 creds = grpc.ssl_channel_credentials(f.read())
             channel = grpc.secure_channel(f'{server}:{port}', creds)
@@ -689,6 +692,9 @@ class NumerousClient:
         if clear:
             self.clear_timeseries_data(self._scenario)
         writer_ = NumerousBufferedWriter(self.write_with_row_generator, self.writer_closed, self._scenario, buffer_size)
+
+        self.writers.append(writer_)
+
         try:
             yield writer_
         finally:
@@ -772,6 +778,10 @@ class NumerousClient:
 
 
     def close(self):
+        #close all writers
+        for w in self.writers:
+            w.close()
+
         if hasattr(self,'log_listener'):
             self.log_listener.stop()
         if self._complete:
